library parsely;

import 'package:parsley/parsing_functions/parse_bool_to_double.dart';
import 'package:parsley/parsing_functions/parse_bool_to_int.dart';
import 'package:parsley/parsing_functions/parse_bool_to_string.dart';
import 'package:parsley/parsing_functions/parse_datetime_to_string.dart';
import 'package:parsley/parsing_functions/parse_double_to_bool.dart';
import 'package:parsley/parsing_functions/parse_double_to_int.dart';
import 'package:parsley/parsing_functions/parse_double_to_string.dart';
import 'package:parsley/parsing_functions/parse_int_list_to_string_list.dart';
import 'package:parsley/parsing_functions/parse_int_to_bool.dart';
import 'package:parsley/parsing_functions/parse_int_to_double.dart';
import 'package:parsley/parsing_functions/parse_int_to_list.dart';
import 'package:parsley/parsing_functions/parse_int_to_string.dart';
import 'package:parsley/parsing_functions/parse_string_list_to_int_list.dart';
import 'package:parsley/parsing_functions/parse_string_to_bool.dart';
import 'package:parsley/parsing_functions/parse_string_to_double.dart';
import 'package:parsley/parsing_functions/parse_string_to_int.dart';
import 'package:parsley/parsing_functions/string_to_date.dart';

T parse<T>(input) {
  return _parsley.parse(input);
}

void registerParseFunction<I, O>(_ParseFunction<I, O> parseFunction) {
  _parsley.registerHandler(parseFunction);
}

typedef O _ParseFunction<I, O>(I input);

final _Parsley _parsley = _Parsley();

class _Parsley {
  final List<_ParseHandler> _parsers = List();

  _Parsley() {
    registerHandler(parseIntToString);
    registerHandler(parseStringToInt);

    registerHandler(parseStringToDouble);
    registerHandler(parseDoubleToString);

    registerHandler(parseStringToBool);
    registerHandler(parseBoolToString);

    registerHandler(parseIntToBool);
    registerHandler(parseBoolToInt);

    registerHandler(parseIntToDouble);
    registerHandler(parseDoubleToInt);

    registerHandler(parseDoubleToBool);
    registerHandler(parseBoolToDouble);

    registerHandler(parseIntToList);
    registerHandler(parseIntListToStringList);
    registerHandler(parseStringListToIntList);

    registerHandler(parseStringToDateTime);
    registerHandler(parseDateTimeToString);
  }

  void registerHandler<I, O>(_ParseFunction<I, O> parseFunction) {
    _parsers.add(_ParseHandler<I, O>(parseFunction));
  }

  T parse<T>(input) {
    if (input == null) {
      return null;
    }
    if (input is T) {
      return input;
    }

    for (_ParseHandler parseHandler in _parsers) {
      if (parseHandler.canParse(input) && parseHandler.canOutput<T>()) {
        try {
          return parseHandler.parse(input);
        } catch (error) {
          throw ParseFunctionException(error);
        }
      }
    }
    throw NoParserFoundException(input);
  }
}

class _ParseHandler<I, O> {
  final _ParseFunction<I, O> parseFunction;

  _ParseHandler(this.parseFunction);

  bool canParseType<J>() {
    return J == I;
  }

  bool canParse(var value) {
    return value is I;
  }

  bool canOutput<T>() {
    return T == O;
  }

  O parse(I i) {
    return parseFunction(i);
  }
}

class ParseFunctionException implements Exception {
  final Exception cause;

  ParseFunctionException(this.cause);
}

class NoParserFoundException implements Exception {
  final dynamic input;

  NoParserFoundException(this.input);
}
